import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;

public class TurnBarrier implements Barrier {
   private final Lock monitor = new ReentrantLock(true);

   private final int n;     // number of expected threads
   private int nDone;       // number of arrived threads
   private int round;       // round of barrier arrivals

   // all threads arrived
   private final Condition allDone = monitor.newCondition();

   // block until expect() threads have reached barrier
   public void await()      // wait is a final method of Object; hence await
        throws InterruptedException {
     monitor.lock();
     try {
       nDone += 1;
       int myRound = round;     // what round am I in?
       if (nDone == n) {        // if last:
         allDone.signalAll();   // signal all
         nDone = 0;             // reset counter
         round = (round+1)%2;   // increase round (modulo 2)
         System.out.println("Turn done!");
       } else {
         while (nDone < n            // else wait for allDone
                && myRound == round) // while the turn
           allDone.await();          // has not changed
       }
     } finally {
       monitor.unlock();
     }
   }

   // number of threads expected at the barrier
   public int expect() {
     return n;   // no need to lock: `n' is final
   }

   // initialize barrier for `n' threads
   TurnBarrier(int n) {
     this.n = n;
   }
}
