import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;

public class MonitorTable implements Table {
	 final private Lock monitor = new ReentrantLock(true);
	 final private int[] forks;
	 final private Condition[] available;

	 public void putForks(int k) {
		  monitor.lock();
		  try {
				forks[left(k)] = -1;
				available[left(k)].signal();
				forks[right(k)] = -1;
				available[right(k)].signal();
		  } finally {
				monitor.unlock();
		  }
	 }

	 public void getForks(int k) {
		  monitor.lock();
		  try {
				while (forks[left(k)] >= 0)
					 available[left(k)].await();
				forks[left(k)] = k;
				while (forks[right(k)] >= 0)
					 available[right(k)].await();
				forks[right(k)] = k;
		  } catch (InterruptedException e) {
				System.out.println("Call to getForks(" + k + ") interrupted.");
		  } finally {
				monitor.unlock();
		  }
	 }

	 public int left(int k) {
		  return k;
	 }

	 public int right(int k) {
		  return (k + 1) % N;
	 }
	 
	 final private int N;

	 MonitorTable(int N) {
		  this.N = N;
		  forks = new int[N];
		  available = new Condition[N];
		  for (int i = 0; i < N; i++) {
				forks[i] = -1;  // forks initially available
				available[i] = monitor.newCondition();
		  }
	 }
}
